#!/usr/bin/env python
# -*- coding: Latin-1 -*-
# vim: tabstop=4 shiftwidth=4 expandtab

""" Miscelanous functions for the Monte Carlo simulations with PyRings

"""

from datetime import datetime, timedelta
import multiprocessing

import numpy as np
from numpy import ma
from numpy.random import random, randn

from rings.ring import Ring
from rings.fitt import carton_uv, carton_scales
from rings.utils import xy2lonlat, lonlat2xy


def synthetic_CLring(x, y, t, cfg):
    """ Creates a synthetic ring like Carton n Legras

        The parameters are passed by a dictionary: cfg
    """
    x_c = cfg['u_c'] * t  # + cfg['xt0']
    y_c = cfg['v_c'] * t  # + cfg['yt0']
    u, v = carton_uv(x-x_c, y-y_c, cfg['omega0'], cfg['delta'], cfg['alpha'])
    return u+cfg['u_c'], v+cfg['v_c']

def drunken_walk(N, step=1, x0=0, y0=0):
    """ Simulates a random walker

    For N iterations, it turns in any direction, equally distributed. Each
    time moves one step
    """
    rad = random(N)*2*np.pi
    dx = ma.sin(rad)*step
    dy = ma.cos(rad)*step
    x = dx.cumsum() + x0
    y = dy.cumsum() + y0
    return x, y

def drunken_drive(N, step=1, x0=0, y0=0):
    """ Simulates a standard normal change in course

    For N iterations it changes the course with a normal probability around
      the previous course. Each time moves one step distance. On the drunken
      drive there is a memory on the course followed, different from the
      drunken walk, and so it is more similar to real sensors (ships, drifters
      etc).

      Variance between -90 to 90 deg.
    """
    rad = 0.25*np.pi*randn(N)
    rad[0] = random(1)*2*np.pi
    rad = rad.cumsum()
    dx = ma.sin(rad)*step
    dy = ma.cos(rad)*step
    x = dx.cumsum() + x0
    y = dy.cumsum() + y0
    return x, y

def random_sample_equal_area(N, Rlimit):
    """ Return x,y for N samples inside the Rlimit.

        The probability is equal to area.
    """
    x = ma.array(Rlimit*(random(N)-0.5)*2)
    y = ma.array(Rlimit*(random(N)-0.5)*2)
    r = (x**2+y**2)**0.5
    ind = np.nonzero(r>Rlimit)[0]
    while ind.any():
        x[ind] = Rlimit*(random(len(ind))-0.5)*2
        y[ind] = Rlimit*(random(len(ind))-0.5)*2
        r = (x**2+y**2)**0.5
        ind = np.nonzero(r>Rlimit)[0]
    return x, y

def error_estimate(cfg):
    N = cfg['montecarlo']['Nsamples'] 
    # Define the (x,y) sampling positions
    x, y = random_sample_equal_area(N , cfg['montecarlo']['Rlimit'] )
    Rmedian = np.median((x**2+y**2)**0.5)
    t = np.arange(N)*cfg['montecarlo']['dt']
    t = t - np.median(t)
    # Estimate the measures
    u, v = synthetic_CLring(x, y, t, cfg['ring'])
    Vmedian = np.median((u**2+v**2)**0.5)
    # Add a noise
    u_noise = cfg['montecarlo']['Vnoise_sigma'] * randn(N)
    v_noise = cfg['montecarlo']['Vnoise_sigma'] * randn(N)
    noisesig_ratio = np.median(
            (u_noise**2 + v_noise**2)**0.5 / \
            (u**2 + v**2)**0.5
            )
    u = u + u_noise
    v = v + v_noise
    # Might have a better way to do the line below.
    d0 = datetime(1,1,1)
    d = d0+ma.array([timedelta(seconds=dt) for dt in t])
    # Transform x,y into lon, lat
    lon, lat = xy2lonlat(x, y, cfg['ring']['lon_t0'], cfg['ring']['lat_t0'])
    # Creating input to Class Ring
    input = {'datetime': d, 'Lon': lon, 'Lat': lat, 'u':u, 'v':v}
    anel = Ring(input)
    # error estimate
    xc_err, yc_err = lonlat2xy(ma.array([anel['Lon_c']]), ma.array([anel['Lat_c']]), [cfg['ring']['lon_t0']], [cfg['ring']['lat_t0']])
    uc_err = anel['uc'] - cfg['ring']['u_c']
    vc_err = anel['vc'] - cfg['ring']['v_c']

    Vmax, Rmax = \
                carton_scales(cfg['ring']['omega0'], cfg['ring']['delta'],
                        cfg['ring']['alpha'])
    output = cfg.copy()
    output['output'] = {'xc_err':xc_err[0], 'yc_err':yc_err[0],
            'uc_err':uc_err, 'vc_err':vc_err, 'Vmax': Vmax, 
            'Rmax': Rmax, 'noisesig_ratio': noisesig_ratio,
            'Rmedian': Rmedian, 'Vmedian':Vmedian}

    return output

def random_cfg(cfg):
    cfg2 = {}
    for k in cfg:
        if type(cfg[k]) == list:
            cfg2[k] = (cfg[k][1]-cfg[k][0])*random(1)[0]+cfg[k][0]
            if (type(cfg[k][0])==int) & (type(cfg[k][1])==int):
                    cfg2[k] = int(cfg2[k])
        elif type(cfg[k]) == dict:
            cfg2[k] = random_cfg(cfg[k])
        else:
            cfg2[k] = cfg[k]
    return cfg2

#import pandas as pd
def montecarlo(cfg_base, N):
    """
    """
    npes = 2 * multiprocessing.cpu_count()
    pool = multiprocessing.Pool(npes)
    results = []

    for n in range(N):
        cfg_tmp = random_cfg(cfg_base)
        # Temporary solution
        #output = error_estimate(cfg_tmp)
        results.append( pool.apply_async( error_estimate, (cfg_tmp,) ) )
    pool.close()

    data = []
    for i, r in enumerate(results):
        output = r.get()
        tmp = {}
        for k in output.keys():
            for kk in output[k].keys():
                tmp[kk] = output[k][kk]
        data.append(tmp)
    pool.terminate()

    #data = pd.DataFrame(data)
    return data



#!/usr/bin/env python
# -*- coding: Latin-1 -*-
# vim: tabstop=4 shiftwidth=4 expandtab

""" A module to handle the fittings for the rings' class
"""

import numpy as np
from numpy import ma

from rings.utils import uv2nt, nt2uv

class v_circular(object):
    """

        Recomendation from the paper reviewer! Include the
          penalty function for the translation speed. Old eq. 7,
          eq. 11 on the new version.
          J = \frac{1}{2N} \sum_{n=1}^{N}
              \frac{{v_{r}^2}_n}{|V_n|}
            + \frac{\lambda}{2} (u_{cn} + v_{cn})^2.

        I'm not sure how would be the best way to include the
          penalty factor lambda. Might be a good idea to depend on
          the number of available observations. More observations,
          less succetible it is to the over fitting effect.
    """
    def __init__(self):
        # Escalas
        self.s = [1e4, 1e4, 1e-2, 1e-2]
        # xc|yc [10 km], uc|vc [1 cm], lambda

        # First guess for the parameters
        # Improve it. Consider the data input to better suggest initial values
        #   like x0 as the median of x, or eta0 as marximum/minimum eta.
        # Consider to migrate for named tuples or dictionary.
        self.p0 = [-1, -1, .1, .1]

    #def set_p0(self, p0=None):
    #    if p0 == None:
    #        self.p0 = [-1, -1, .1, .1]

    def cost(self, p, dt, x, y, u, v):
        """
        """
        # I'm not sure this is the best way to count the number of elements.
        n = len(u.compressed().flatten())

        s = self.s
        u_new = u - s[2]*p[2]
        v_new = v - s[3]*p[3]
        x_new = x - dt*s[2]*p[2]
        y_new = y - dt*s[2]*p[3]
        vr, vt = uv2nt(x_new, y_new, u_new, v_new, x_c=s[0]*p[0], y_c=s[1]*p[1])
        mag = (u**2+v**2)**0.5
        #vt = -u*ma.sin(ma.arctan2(y-p[1], x-p[0])) + \
        #      v*ma.cos(ma.arctan2(y-p[1], x-p[0]))
        #e = 1./(2*n)*ma.sum( (vr/mag)**2 )
        # Penalized version
        # Maybe
        #e = 1./(2*n)*ma.sum( vr**2/mag )
        #e = 1./n*ma.sum( vr**2 )

        # I need to do a better job defining the lambda. It's working well,
        #   but I feel like it could be improved.
        lam = 1e-2
        e = 1./(2*n)*ma.sum( (vr/mag)**2 ) + \
                lam/2 * (s[2]*p[2]**2 + s[3]*p[3]**2)
        return e

class v_circular_nontranslating(object):
    def __init__(self):
        # Escalas
        self.s = [1e4, 1e4]

    def cost(self, p, x, y, u, v):
        """
        """
        # I'm not sure this is the best way to count the number of elements.
        n = len(u.compressed().flatten())

        s = self.s
        vr, vt = uv2nt(x, y, u, v, x_c=s[0]*p[0], y_c=s[1]*p[1])
        mag = (u**2+v**2)**0.5
        #vt = -u*ma.sin(ma.arctan2(y-p[1], x-p[0])) + \
        #      v*ma.cos(ma.arctan2(y-p[1], x-p[0]))
        e = 1./n*ma.sum( (vr/mag)**2 )
        #e = 1./n*ma.sum( vr**2 )
        return e

# =========================
# Moving carton_uv to here just to be able to run the Monte Carlo tests,
#   but in the future, create a new class for the Carton&Legras vortex.
def carton_uv(x, y,omega0, delta, alpha):
    """ Carton's model for azimuthal velocity
    """
    r = ((x)**2+(y)**2)**0.5
    mag = 0.5*omega0*(((x)**2+(y)**2)**0.5)*np.exp(-((((x)**2+(y)**2)**0.5)/delta)**alpha)
    u,v = nt2uv(x,y,0,mag)
    return u, v

def carton_scales(omega0, delta, alpha):
    r = np.arange(0, 400e3, 5e3)
    v = 0.5*omega0*(r)*np.exp(-(r/delta)**alpha)
    Vmax = v.max()
    Rmax = r[v.argmax()]
    return Vmax, Rmax



#!/usr/bin/env python
# -*- coding: Latin-1 -*-
# vim: tabstop=4 shiftwidth=4 expandtab

"""Ocean Rings related stuff.

     Atention!!! Work on variables _r. Maybe I should use instead of _r
       for the ring variables, use _raw for the input, or a dictionary
       data_intput, and use x,r,u,v for the data corrected for the ring.


    Should strongly consider to use namedtuple, for efficiency and clarity
      purposes.
    http://docs.python.org/library/collections.html#collections.namedtuple

    Saving this notes from a modified version of the paper.
    %\note{Point the solid body inner core, hence CL would be a good model.}
    %
    %\citet{CartonLegras1994}
    %
    %\begin{displaymath}
    %    v_{\theta CL} = \frac{1}{2} \omega_0 r \exp \left( - \left( \frac{r}{\delta} \right)^\alpha \right)
    %\end{displaymath}
    %\begin{displaymath}
    %    %J = \frac{1}{2N} \sum_{n=1}^{N} \left( v_{\theta CL \ n} - v_{\theta n} \right)^2
    %    J = \frac{1}{2N} \sum \left( v_{\theta CL} - v_{\theta} \right)^2
    %    %J = \frac{1}{2N} \sum \left( v_{\theta CL}^2 + v_r^2 - |V|^2 \right)^2
    %\end{displaymath}


    Another idea, include uncertain on the data. That would allow to use
      different datasets and give different weights for each type of data.
"""

#import logging
from datetime import datetime, timedelta

import numpy as np
from numpy import ma
from scipy import optimize

from rings.utils import uv2nt
from rings import fitt
#from fluid.common.common import lonlat2xy, xy2lonlat
from rings.utils import lonlat2xy, xy2lonlat


class Ring(object):
    """ A class to deal with Rings

        Maybe should split in two classes, one for eta inputs and other for
          velocities input.

        Input:
          data:
            Lat: Latitude
            Lon: Longitude
            datetime: Time [datetime]
            u: u component of velocity
            v: v component of velocity
    """

    def __init__(self, input, metadata={}, auto=True, **keywords):
        """
        """

        self.input = input
        self.data = {}
        self.metadata = metadata

        self._set_default_values(keywords)

        self.set_xy()
        self.set_t()

        self.go()
        self.set_xy()   # Redefine the positions, discounting the uc|vc
        self.set_cilyndrical_components()


    def keys(self):
        k = list(self.data.keys())
        k.extend(self.input.keys())
        return k

    def __getitem__(self, key):
        try:
            return self.input[key]
        except:
            return self.data[key]

    def _set_default_values(self, keywords):
        """
        """

        if 'center' in keywords:
            self.center = keywords['center']
        else:
            self.center = {}

        # Lat, Lon of the origin of the coordinate system, i.e.  (x,y)
        if (not hasattr(self, 'lat_ref')) & (not hasattr(self, 'lon_ref')):
            try:
                self.lat_ref = self.center['lat']
                self.lon_ref = self.center['lon']
            #except AttributeError:
            except KeyError:
                self.lat_ref = ma.median(self.input['Lat'])
                self.lon_ref = ma.median(self.input['Lon'])

    def set_xy(self):
        """ Set x, y coordinates from Lat, Lon.

            Most of the procedures are based on cartesian distances. This
              function define x and y distance from lat_ref, lon_ref.
        """

        self.data['x'], self.data['y'] = lonlat2xy(self.input['Lon'],
                                         self.input['Lat'],
                                         lat_0=self.lat_ref,
                                         lon_0=self.lon_ref)

        #if ('u' in self.center) & ('v' in self.center):
        #    self['x_r'] = self['x']-self.center['u']*self['dt']
        #    self['y_r'] = self['y']-self.center['v']*self['dt']

    def set_t(self):
        """ Create an array t as seconds from self.t_ref

            If t_ref is not defined, it uses the median of the input['datetime'],
              than create an array data['t'] with the seconds relative to t_ref.

            It's not the most efficient way, but is done to work well with any
              datetime dimension.
        """
        t0 = min(self['datetime'])
        dt = ma.array([dt.total_seconds() for dt in self['datetime'] - t0])
        dt_median = ma.median(dt)
        self.t_ref = t0+timedelta(seconds = dt_median)
        self.data['t'] = dt-dt_median

    def set_cilyndrical_components(self):
        self.data['r'] = (self.data['x']**2 + self.data['y']**2)**0.5
        self.data['vrad'], self.data['vtan'] = uv2nt(self['x'], self['y'],
                self['u'], self['v'], x_c=self['xc'], y_c=self['yc'])

    def go(self):
        """
        """
        args = (self.data['t'], self.data['x'], self.data['y'],
                self.input['u'], self.input['v'])
        f = fitt.v_circular()
        op = optimize.fmin_tnc(f.cost, f.p0, fprime=None,  args=args, approx_grad=1,  bounds = None, epsilon=1e-08, scale=None, offset=None, messages=15, maxCGit=-1, maxfun=500, eta=-1, stepmx=0, accuracy=0, fmin=0, ftol=-1, xtol=-1, pgtol=-1, rescale=-1, disp=5)
        p = op[0]   # Output fitting parameters
        self.data['xc'] = f.s[0]*p[0]
        self.data['yc'] = f.s[1]*p[1]
        self.data['uc'] = f.s[2]*p[2]
        self.data['vc'] = f.s[3]*p[3]

        self.data['xr'] = self.data['x'] - self.data['t']*self.data['uc']
        self.data['yr'] = self.data['y'] - self.data['t']*self.data['vc']
        self.data['ur'] = self.input['u'] - self.data['uc']
        self.data['vr'] = self.input['v'] - self.data['vc']

        self.data['Lon_c'], self.data['Lat_c'] = xy2lonlat(self.data['xc'],
                self.data['yc'], self.lon_ref, self.lat_ref)

    def plot(self):
        import pylab
        #import pdb; pdb.set_trace()
        pylab.quiver(self.data['x'], self.data['y'], self.input['u'],
                self.input['v'], color='r')
        pylab.quiver(self.data['xr'], self.data['yr'], self.data['ur'], self.data['vr'])
        pylab.plot(self.data['xc'], self.data['yc'], 'ro')
        pylab.figure()
        pylab.plot(self.data['r'], self.data['vtan'], 'r')
        pylab.plot(self.data['r'], self.data['vrad'], 'g')
        pylab.show()


# ============================================================================
def center_velocity_correction(dt, x, y, u, v, u_c, v_c):
    """Correct data due ring propagation velocity

    Input:
        -> u => Zonal velocity [m/s]
        -> v => Meridional velocity [m/s]
        -> u_c => Ring center zonal velocity [m/s]
        -> v_c => Ring center meridional velocity [m/s]

    Subtract the velocity field by the ring propagation and
      move the data position according to the ring propagation,
      so estimate what should be the sample if was done on one
      instant.
    """
    # Correct velocity field from center movement
    u_new = u-u_c
    v_new = v-v_c

    # Corrected positions due center movement
    x_new=x-dt*u_c
    y_new=y-dt*v_c

    return x_new, y_new, u_new, v_new




#!/usr/bin/env python
# -*- coding: Latin-1 -*-
# vim: tabstop=4 shiftwidth=4 expandtab

""" Script to run Monte Carlo simulations with PyRings.

    The misc directory is probably a better place for this code.

"""

from datetime import datetime, timedelta

import numpy as np
from numpy import ma

from rings.misc.montecarlo import *
#from rings.misc.montecarlo import montecarlo

# ============================================================================
cfg_base = {'ring':
            {'omega0': [1.7e-5, 2.7e-5], 
                'delta': [120e3, 200e3], #170e3, 
                'alpha': 2.5,
                'u_c': [-0.25, 0.25],
                'v_c': [-0.25, 0.25],
                'lat_t0': 8,
                'lon_t0': -50},
            'montecarlo':{
                'dt': 0,
                'Vnoise_sigma': 0, #[0.0, 0.20],
                'Nsamples': [25, 1000], 
                'Rlimit': [25e3, 300e3]
            }
        }


data = montecarlo(cfg_base, 100)
import pandas as pd
data = pd.DataFrame(data)
data.describe()


store = pd.HDFStore('montecarlo.h5')
store.append('all_snapshot', data)

#data['Lc_err'] = (data['xc_err']**2 + data['yc_err']**2)**0.5
#data['Vc_err'] = (data['uc_err']**2 + data['vc_err']**2)**0.5



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
#from distutils.core import setup

import os
import sys
from distutils import log

here = os.path.abspath(os.path.dirname(__file__))
README = "." #open(os.path.join(here, 'README.rst')).read()
NEWS = "." #open(os.path.join(here, 'NEWS.txt')).read()

install_requires=[
    "numpy >= 1.1", 
    "scipy >= 0.10.1",
    "pandas >= 0.10.1",
    "fluid >= 0.1.10",
],

version = '0.7.6'

setup(
    name = 'pyrings',
    version = version,
    description = "Procedures to handle coherent vortexes in the ocean",
    long_description=README + '\n\n' + NEWS,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Python Software Foundation License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],
    keywords='rings, eddies, ADCP, Gradient Balance',
    author='Guilherme Castelao, Luiz Irber, Ana Villas Boas',
    author_email='guilherme@castelao.net, luiz.irber@gmail.com',
    url='http://pyrings.castelao.net',
    license='PSF',
    #download_url="https://github.com/castelao/pyrings/archive/master.zip",
    #download_url="https://pypi.python.org/packages/source/r/rings/",
    download_url="https://pypi.python.org/packages/source/p/pyrings/",
    #py_modules=['rings.rings','rings_plots','rings.okuboweiss','okuboweiss_plot','rings.EddyTracking'],
    #py_modules=['rings.ring', 'rings.utils', 'rings.fitt','rings.misc.montecarlo'],
    #packages=find_packages(),
    packages=['rings', 'rings.misc', 'rings.test'],
    scripts=['bin/run_montecarlo.py',],
    zip_safe=True,
    install_requires=install_requires,
    platforms = ['any']
)



#!/usr/bin/env python
# -*- coding: Latin-1 -*-
# vim: tabstop=4 shiftwidth=4 expandtab

""" Util functions for the module rings
"""

import ConfigParser
import re
from datetime import timedelta

import numpy
from numpy import ma

# ============================================================================
def lonlat2xy(lon,lat,lon_0=None,lat_0=None):
    """ Convert pairs of (Lat,Lon) into (x,y)

        Input:
                      Lon [deg]
          Lat [deg]
          Lon_0 [deg] => Lon of the origin of the cartesian system
          Lat_0 [deg] => Lat of the origin of the cartesian system
        Output:
                      x [m]
          y [m]

        The projection is deformed as get away from the center. Since the
          Latitudes don't deform, the y is estimated first, then for each
          point is estimated the distante to the meridian of reference
          (Lon_0) considering the Latitude of the measurement.
    """
    if (lat_0==None) or (lon_0==None):
        lat_0=numpy.median(lat)
        lon_0=numpy.median(lon)
    from fluid.common.distance import distance
    y=distance(lat,0,lat_0,0)
    y[lat<lat_0]=-1*y[lat<lat_0]
    x=distance(lat,lon,lat,lon_0)
    x[lon<lon_0]=-1*x[lon<lon_0]
    return x,y

# ============================================================================
def xy2lonlat(x,y,lon_0,lat_0):
    """ Think how to improve it.1
    """
    DEG2RAD = (2*numpy.pi/360)
    RAD2DEG = 1/DEG2RAD
    DEG2MIN = 60.
    DEG2NM  = 60.
    NM2M   = 1852.    # Defined in Pond & Pickard p303.
    lat = lat_0+y/(DEG2NM*NM2M)
    fac = numpy.cos((lat+lat_0)/2.*numpy.pi/180)
    lon = lon_0+x/(DEG2NM*NM2M)/fac
    return lon, lat

# ============================================================================
def uv2nt(x,y,u,v,x_c=0,y_c=0):
    """Convert orthogonal velocity components to normal and tangent ones

    Based on x_c and y_c, with default values (0,0) convert u and v to
    normal (n) and tangent(t) velocities.
    Input:
        - u =>
        - v =>
        - x =>
        - y =>
        - x_c =>
        - y_c =>
    Output:
        - n =>
        - t =>
    """
    dx = x-x_c
    dy = y-y_c
    #
    alpha = numpy.arctan2(dy,dx) # Angle between x-axis and a rotated x passing throught the velocity point
    #
    n = u*numpy.cos(alpha)+v*numpy.sin(alpha)
    t = v*numpy.cos(alpha)-u*numpy.sin(alpha)
    #
    return [n,t]

# ============================================================================
def nt2uv(x,y,n,t,x_c=0,y_c=0):
    """Convert normal and tangent velocities to Orthogonal ones

    Convert normal(n) and tangent(t) velocities to orthogonal ones,
    u and v, based on x,y of each (n,t) velocity and the circular center
    x_c and y_c
    Input:
        - n =>
        - t =>
        - x =>
        - y =>
        - x_c =>
        - y_c =>
    Output:
        - u =>
        - v =>
    """

    dx = x-x_c
    dy = y-y_c

    alpha = numpy.arctan2(dy,dx) # Angle between x-axis and a rotated x passing throught the velocity point

    u = n*numpy.cos(alpha) - t*numpy.sin(alpha);
    v = n*numpy.sin(alpha) + t*numpy.cos(alpha);

    return [u,v]


# ============================================================================
# ==== Logging System
# ============================================================================

# This basic logger should be out of here. At most, in a miscellaneous file.

def basic_logger(logname="Generic", logfile=None):
    """ Create a logger

        !!!ATENTION!!!
        In the future change it to use a config file, which would give 
          far away more flexibility.
    """
    import logging
    import logging.handlers

    # Creating another log level
    logging.VERBOSE = logging.DEBUG - 1
    logging.addLevelName(logging.VERBOSE, 'VERBOSE')

    #create logger
    logger = logging.getLogger(logname)
    #logger.setLevel(logging.DEBUG)
    logger.setLevel(logging.VERBOSE)

    #create console handler and set level to debug
    ch = logging.StreamHandler()
    #ch.setLevel(logging.WARN)
    ch.setLevel(logging.INFO)
    #create formatter
    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s -  %(message)s")
    #add formatter to ch
    ch.setFormatter(formatter)
    #add ch to logger
    logger.addHandler(ch)

    if logfile is not None:
        #create a rotate file handler
        fh = logging.handlers.RotatingFileHandler(logfile,
               mode='a', maxBytes=1000000, backupCount=10)
        fh.setLevel(logging.DEBUG)
        #ch.setLevel(logging.WARN)
        #create formatter
        formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s -  %(message)s")
        #add formatter to ch
        fh.setFormatter(formatter)
        #add ch to logger
        logger.addHandler(fh)

    #"application" code
    #logger.debug("debug message")
    #logger.info("info message")
    #logger.warn("warn message")
    #logger.error("error message")
    #logger.critical("critical message")

    return logger


# ============================================================================
# ==== Auxiliary functions
# ============================================================================

def cfg2dict(cfg_file):
    """Extract all variables from a ConfigParser file and create a dict[section][option]

    """
    cfg = ConfigParser.SafeConfigParser()
    try:
        cfg.readfp(open(cfg_file))
    except:
        print "Couldn't open the config file: %s" % cfg_file
        print "Improve this error handle"
        return
    parameters = {}
    for section in cfg.sections():
        parameters[section] = {}
        for option in cfg.options(section):
            par = cfg.get(section, option)
            if option == 'w0':
                option = 'W0'
            if re.match("^\ *\[(.*)\]\ *$",par):
                l = re.search("^\ *\[(.*)\]\ *$",par).group(1)
                tmp = []
                for i in re.split(',',l):
                    if re.match("'.*'",i):
                        tmp.append(re.search("'(.*)'",i).group(1))
                    elif re.match("^-?\d+$",i):
                        tmp.append(int(i))
                    elif re.match("^-?\d+\.\d*$",i):
                        tmp.append(float(i))
                parameters[section][option] = tmp
                del(l,tmp,i)
            #if re.search("\[('(\w+)')?((,\ ?'(\w+)')?)+\]",par):
            #    parameters[section][option] = re.findall("'(\w+)'",par)
            #elif re.search("^\[(\[(\w+)(,\ ?\w+)*\](,\ )?)+\]$",par):
            #    parameters[section][option] = [] 
            #    for x in re.findall("\[\w+(?:,\ ?\w+)*\]",par):
            #        parameters[section][option]
            elif re.search("^True$",par):
                parameters[section][option] = True
            elif re.search("^False$",par):
                parameters[section][option] = False
            elif re.search("^-?\d+$",par):
                parameters[section][option] = int(par)
            elif re.search("^-?\d+(\.\d+)?(e-?\d+)?$", par):
                parameters[section][option] = float(par)
            elif re.match('^(?P<value>\d+)(?P<unit>days|hours|minutes|seconds)$',par):
                tmp = re.match('^(?P<value>\d+)(?P<unit>days|hours|minutes|seconds)$',par)
                if tmp.group('unit')=='days':
                    parameters[section][option] = timedelta(days=float(tmp.group('value')))
            else:
                parameters[section][option] = cfg.get(section,option)
    ks = parameters.keys()
    for k in ks:
        if re.search('\w+\.(?:\w+)+',k):
            kk=re.split('\.',k)
            # Stupid way not generic
            if kk[0] not in parameters:
                parameters[kk[0]] = {}
            if kk[1] not in parameters[kk[0]]:
                parameters[kk[0]][kk[1]] = {}
            if len(kk)==2:
                for kkk in parameters[k].keys():
                    parameters[kk[0]][kk[1]][kkk]=parameters[k][kkk]
            elif len(kk)==3:
                parameters[kk[0]][kk[1]][kk[2]]=parameters[k]
            del(parameters[k])
    # ------- Default values
    # I need to think about an efficient way to load default values.
    # ---- Eddy
    if 'eddy' not in parameters:
        parameters['eddy'] = {}
    if 'force_reprocess' not in parameters['eddy']:
        parameters['eddy']['force_reprocess'] = False
    # ---- Eddy Tracking system
    if 'eddy_track' not in parameters:
        parameters['eddy_track'] = {}
    if 'force_reprocess' not in parameters['eddy_track']:
        parameters['eddy_track']['force_reprocess'] = False
    return parameters

# ----------------------------------------------------------------------------


class EddyDataVar(object):
    """ Return each variable for each eddy

        This is necessary because the reference data fields are
          stored as dictionaries of 3D data:
          {var1: 3D MA, var2: 3D MA}
          but with this class is possible to recover the data as
          EddyDataVar[var1]
    """
    def __init__(self, input, ind, mask=False):
        """
        """
        self.input = input
        self.ind = ind
        self.mask = mask

    def keys(self):
        return self.input.keys()

    def __getitem__(self,key):
        if key in ['datetime']:
            # Non grided will fail here. Since it'll not be a slice
            return self.input[key][self.ind[0]][0]
        elif key in ['Lat', 'Lon']:
            tmp = self.input[key][self.ind[-2:]]
            return ma.masked_array(tmp.data, ((tmp.mask==True) | (self.mask==True)))
        else:
            tmp = self.input[key][self.ind][0]
            return ma.masked_array(tmp.data, ((tmp.mask==True) | (self.mask==True)))


class EddyData(object):
    """
    """
    def __init__(self, input):
        """
        """
        #self.data = {}
        self.input = input
        #for id in ids:
        #    ind = ma.nonzero(self.input['patches']==id)
        #    self.data[id] = {}
        #    self.data[id]['ssh'] = EddyDataVar(self.input, ind)

    def keys(self):
        return self.input.keys()

    @property
    def ids(self):
        return ma.unique(self.input['patches'].compressed())

    def __getitem__(self, id):
        """
        """
        ind = ma.nonzero(self.input['patches']==id)
        gridded = True
        if gridded == True:
            ind = slice(ind[0].min(), ind[0].max()+1), \
              slice(ind[1].min(), ind[1].max()+1), \
              slice(ind[2].min(), ind[2].max()+1)
            mask = ((self.input['patches'][ind].data!=id) | self.input['patches'][ind].mask)
            return EddyDataVar(self.input, ind, mask)
        else:
            return EddyDataVar(self.input, ind)

    def getaround(self, id, bl=2):
        """
        """
        ind = ma.nonzero(self.input['patches']==id)
        gridded = True
        if gridded == True:
            ind = slice(ind[0].min(), ind[0].max()+1), \
              slice(max(0,ind[1].min()-bl), ind[1].max()+1+bl), \
              slice(max(0,ind[2].min()-bl), ind[2].max()+1+bl)
            return EddyDataVar(self.input, ind)
        else:
            return EddyDataVar(self.input, ind)

# ============================================================================
def neighbor_cluster(ind, verbose=False):
    """ Define neighbor patches

        The input is a boolean array with True on
          the values of interest. The output is an
          integer array, with one number for each
          group. I.e. all gridpoints with 10 are
          part of the same patch, the patch #10.

       ATENTION:
        Think about it. Maybe I should consider as neighbors only the
          NSEW, and don't include the diagonals.
        Maybe I should consider a jumping cell, i.e. if a neighbor is W<0
          and W<|W0|, and the next one is OK, should take all them as one group. No,
          this case should be dealed with filters. Smooth the data before group it,
          so I have control on the scales that I'm manipulating.
    """
    output = ma.masked_all(ind.shape, dtype='i')
    I,J = numpy.nonzero(ind)
    g = -1
    while I.any():
        if verbose:
            print "Missing %s gridpoints to evaluate" % (I.shape[0])
        g += 1
        gi, gj = [I[0]], [J[0]]
        I, J = I[1:], J[1:]
        while len(gi)>0:
            i, j = gi[0], gj[0]
            ind_around = numpy.nonzero(((I==i-1) & (J==j)) |
              ((I==i+1) & (J==j)) | ((I==i) & (J==j-1)) |
              ((I==i) & (J==j+1)))[0]
            for n in ind_around:
                gi.append(I[n])
                gj.append(J[n])
            I = numpy.delete(I, ind_around)
            J = numpy.delete(J, ind_around)
            output[gi[0],gj[0]] = g
            gi, gj = gi[1:], gj[1:]
    return output

# ============================================================================


def vortex_3Dpatches(data,logger=None):
    """
    """
    id3d = ma.masked_all(data['id'].shape,dtype='i')
    id3d[0]=data['id'][0]
    for zn in range(1,data['id'].shape[0]):
        for id in set(data['id'][zn].compressed()):
            eddy_index=data['id'][zn]==id
            possible=[p for p in set((id3d[zn-1][eddy_index]).compressed())]
            if len(possible)==0:
                id3d.data[zn][eddy_index]=(id3d.max()+1)
                id3d.mask[zn][eddy_index]=False
            elif len(possible)>0:
                import rpy2.robjects as robjects
                criteria={'p':ma.masked_all(len(possible)),'cor':ma.masked_all(len(possible)),'conf.min':ma.masked_all(len(possible)),'n':ma.masked_all(len(possible),dtype='i')}
                for pn,p in enumerate(possible):
                    #possible_index=output['id'][zn-1]==p
                    possible_index=id3d[zn-1]==p
                    index = possible_index & eddy_index
                    # ----
                    # Check if the common area is at least half of the smaller eddy level
                    if (len(speed[zn][index])) < 0.65*(min(len(speed[zn-1][possible_index]),len(speed[zn][eddy_index]))):
                        criteria['p'][pn]=p
                        criteria['cor'][pn]=0
                        criteria['conf.min'][pn]=0
                    else:
                        c = robjects.r['cor.test'](robjects.FloatVector(speed[zn-1][index]),robjects.FloatVector(speed[zn][index]))
                        criteria['p'][pn]=p
                        criteria['cor'][pn]=c.r['estimate'][0][0]
                        criteria['conf.min'][pn]=c.r['conf.int'][0][0]
                        criteria['n'][pn] = len(speed[zn][index])
                tmp=(criteria['cor']>.85) & (criteria['conf.min']>.65)
                if tmp.any():
                    possible=criteria['p'][tmp]
                    if len(possible)==1:
                        id3d.data[zn][eddy_index]=possible[0].astype('i')
                        id3d.mask[zn][eddy_index]=False
                    else:
                        theone = possible.min()
                        id3d.data[zn][eddy_index]=theone
                        id3d.mask[zn][eddy_index]=False
                        for pp in [p for p in possible if p!=theone]:
                            id3d.data[id3d==pp]=theone
                else:
                    id3d.data[zn][eddy_index]=(id3d.max()+1)
                    id3d.mask[zn][eddy_index]=False
    return id3d

# ============================================================================
# ============================================================================


def prepare_masked_array(data, float_fillvalue=1e20, int_fillvalue=-999999):
    """ Prepare masked array to be saved.

        Force data os masked elements to be a certain value

            # Be sure to set all masked values to 1e+20, so the masked 
            # array can be recreated once read.
            # This happens because the masked_all just put a bogus data in .data, so
            #   who understands the masked array goes fine, otherwise.
    """
    for k in data.keys():
        if type(data[k]) == numpy.ma.core.MaskedArray:
            if data[k].dtype in ('>f4', 'float32', '>f8', 'float64'):
                if data[k].mask.any():
                    data[k].data[data[k].mask == True] = float_fillvalue
                data[k].set_fill_value(float_fillvalue)
            elif data[k].dtype in ('int32',):
                if data[k].mask.any():
                    data[k].data[data[k].mask == True] = int_fillvalue
                data[k].set_fill_value(int_fillvalue)
    return data
